@@ -35,7 +35,7 @@
 //  Global Variables  //
 ////////////////////////
 const char *util_name = "openSeaChest_NVMe";
-const char *buildVersion = "2.2.2";
+const char *buildVersion = "2.3.0";
 
 ////////////////////////////
 //  functions to declare  //
@@ -89,7 +89,7 @@ int32_t main(int argc, char *argv[])
     GET_TELEMETRY_VAR
     TELEMETRY_DATA_AREA_VAR
     OUTPUT_MODE_VAR
-    GET_FEATURES_VAR
+    GET_FEATURES_VARS
     NVME_TEMP_STATS_VAR
     NVME_PCI_STATS_VAR
     MODEL_MATCH_VARS
@@ -112,7 +112,6 @@ int32_t main(int argc, char *argv[])
     int argIndex = 0;
     int optionIndex = 0;
 
-    //add -- options to this structure DO NOT ADD OPTIONAL ARGUMENTS! Optional arguments are a GNU extension and are not supported in Unix or some compilers- TJE
     struct option longopts[] = {
         //common command line options
         DEVICE_LONG_OPT,
@@ -225,17 +224,20 @@ int32_t main(int argc, char *argv[])
             }
             else if (strcmp(longopts[optionIndex].name, GET_FEATURES_LONG_OPT_STRING) == 0)
             {
-                if (isdigit(optarg[0]))
+                uint64_t temp = 0;
+                GET_FEATURES_FLAG = true;
+                if (strncmp(optarg, "help", strlen(optarg)) == 0)
                 {
-                    GET_FEATURES = C_CAST(uint8_t, atoi(optarg));
-                }
-                else if (strncmp(optarg, "help", strlen(optarg)) == 0)
-                {
-                    GET_FEATURES = 0;
+                    nvme_Print_Feature_Identifiers_Help();
+                    exit(UTIL_EXIT_NO_ERROR);
                 }
                 else if (strncmp(optarg, "list", strlen(optarg)) == 0)
                 {
-                    GET_FEATURES = 0x0E;//Using a reserved value - Revisit later 
+                    GET_FEATURES = UINT16_MAX;//set a value higher than is possible to request to know it is invalid and requesting the list
+                }
+                else if (get_And_Validate_Integer_Input(optarg, &temp))
+                {
+                    GET_FEATURES = C_CAST(uint8_t, temp);
                 }
                 else
                 {
@@ -254,7 +256,7 @@ int32_t main(int argc, char *argv[])
             }
             else if (strcmp(longopts[optionIndex].name, DOWNLOAD_FW_LONG_OPT_STRING) == 0)
             {
-                int scanRet = sscanf(optarg, "%s", DOWNLOAD_FW_FILENAME_FLAG);
+                int scanRet = sscanf(optarg, FIRMWARE_FILE_NAME_MAX_LEN_FORMAT_STR, DOWNLOAD_FW_FILENAME_FLAG);
                 if (scanRet > 0 && scanRet != EOF)
                 {
                     DOWNLOAD_FW_FLAG = true;
@@ -613,7 +615,7 @@ int32_t main(int argc, char *argv[])
 
     if (LICENSE_FLAG)
     {
-        print_EULA_To_Screen(false, false);
+        print_EULA_To_Screen();
     }
 
     if (SCAN_FLAG || AGRESSIVE_SCAN_FLAG)
@@ -748,7 +750,11 @@ int32_t main(int argc, char *argv[])
         }
         exit(UTIL_EXIT_INVALID_DEVICE_HANDLE);
     }
-
+    //need to make sure this is set when we are asked for SAT Info
+    if (SAT_INFO_FLAG)
+    {
+        DEVICE_INFO_FLAG = goTrue;
+    }
     //check that we were given at least one test to perform...if not, set that we are dumping device information so we at least do something
     if (!(DEVICE_INFO_FLAG
           || LOWLEVEL_INFO_FLAG
@@ -760,7 +766,7 @@ int32_t main(int argc, char *argv[])
           || (TRANSITION_POWER_STATE_TO >= 0)
           || (GET_NVME_LOG_IDENTIFIER > 0) // Since 0 is Reserved
           || (GET_TELEMETRY_IDENTIFIER > 0)
-          || (GET_FEATURES < UINT8_MAX)
+          || (GET_FEATURES_FLAG)
           || EXT_SMART_LOG_FLAG1
           || CLEAR_PCIE_CORRECTABLE_ERRORS_LOG_FLAG
           || NVME_TEMP_STATS_FLAG
@@ -1068,26 +1074,36 @@ int32_t main(int argc, char *argv[])
             }
         }
 
-        if (GET_FEATURES < UINT8_MAX)
+        if (GET_FEATURES_FLAG)
         {
-            if (GET_FEATURES == 0) //help
-            {
-                nvme_Print_Feature_Identifiers_Help();
-            }
-            else if (GET_FEATURES == 0x0E) //List them all
+            if (GET_FEATURES > UINT8_MAX) //List them all
             {
                 nvme_Print_All_Feature_Identifiers(&deviceList[deviceIter], NVME_CURRENT_FEAT_SEL, false);
             }
             else
             {
                 //Get the feature
-                if (nvme_Print_Feature_Details(&deviceList[deviceIter], GET_FEATURES, NVME_CURRENT_FEAT_SEL) != SUCCESS)
+                switch (nvme_Print_Feature_Details(&deviceList[deviceIter], C_CAST(uint8_t, GET_FEATURES), NVME_CURRENT_FEAT_SEL))
                 {
+                case SUCCESS:
+                    break;
+                case NOT_SUPPORTED:
+                    //This should really only be a "device does not support this", but there are a lot of features we cannot parse details
+                    //for at this time, so for now this message is a bit more of a catch-all
+                    if (VERBOSITY_QUIET < toolVerbosity)
+                    {
+                        printf("Either the requested feature is not supported by the device, or the\n");
+                        printf("utility is unable to show details about the request feature at this time.\n");
+                    }
+                    exitCode = UTIL_EXIT_OPERATION_NOT_SUPPORTED;
+                    break;
+                default:
                     if (VERBOSITY_QUIET < toolVerbosity)
                     {
                         printf("ERROR: failed to get details for feature id %d\n", GET_FEATURES);
                     }
                     exitCode = UTIL_EXIT_OPERATION_FAILURE;
+                    break;
                 }
             }
         }
@@ -1154,7 +1170,7 @@ int32_t main(int argc, char *argv[])
                             {
                                 if (VERBOSITY_QUIET < toolVerbosity)
                                 {
-                                    printf("Error: Unknown/Unsupported output mode %" PRIu8 "\n", OUTPUT_MODE_IDENTIFIER);
+                                    printf("Error: Unknown/Unsupported output mode %d\n", OUTPUT_MODE_IDENTIFIER);
                                 }
                                 exitCode = UTIL_EXIT_OPERATION_FAILURE;
                             }
@@ -1313,6 +1329,7 @@ int32_t main(int argc, char *argv[])
                     cmdOpts.addr = logBuffer;
                     cmdOpts.dataLen = size;
                     cmdOpts.lid = GET_TELEMETRY_IDENTIFIER;
+                    cmdOpts.lsp = 0x01; // FDCHEST-282 Fix
                     cmdOpts.offset = offset;
 
                     rtnVal = nvme_Get_Log_Page(&deviceList[deviceIter], &cmdOpts);
@@ -1357,6 +1374,7 @@ int32_t main(int argc, char *argv[])
                                 cmdOpts.addr = logBuffer;
                                 cmdOpts.dataLen = size;
                                 cmdOpts.lid = GET_TELEMETRY_IDENTIFIER;
+                                cmdOpts.lsp = 0x01; // FDCHEST-282 Fix
                                 cmdOpts.offset = offset;
 
                                 rtnVal = nvme_Get_Log_Page(&deviceList[deviceIter], &cmdOpts);
@@ -1468,11 +1486,18 @@ int32_t main(int argc, char *argv[])
             {
             case SUCCESS:
                 //nothing to print here since if it was successful, the log will be printed to the screen
+                break;
+			case NOT_SUPPORTED:
+				if (VERBOSITY_QUIET < toolVerbosity)
+                {
+                    printf("Printing Temperature Statistics data is not supported on this drive type.\n");
+                }
+                exitCode = UTIL_EXIT_OPERATION_NOT_SUPPORTED;
                 break;
             default:
                 if (VERBOSITY_QUIET < toolVerbosity)
                 {
-                    printf("A failure occured while trying to get Error Information Log\n");
+                    printf("A failure occured while trying to get Temp Statistics Information Log\n");
                 }
                 exitCode = UTIL_EXIT_OPERATION_FAILURE;
                 break;
@@ -1485,11 +1510,18 @@ int32_t main(int argc, char *argv[])
             {
             case SUCCESS:
                 //nothing to print here since if it was successful, the log will be printed to the screen
+                break;
+			case NOT_SUPPORTED:
+				if (VERBOSITY_QUIET < toolVerbosity)
+                {
+                    printf("Printing PCI Statistics data is not supported on this drive type.\n");
+                }
+                exitCode = UTIL_EXIT_OPERATION_NOT_SUPPORTED;
                 break;
             default:
                 if (VERBOSITY_QUIET < toolVerbosity)
                 {
-                    printf("A failure occured while trying to get Error Information Log\n");
+                    printf("A failure occured while trying to get PCI Statistics Information Log\n");
                 }
                 exitCode = UTIL_EXIT_OPERATION_FAILURE;
                 break;
@@ -2108,6 +2140,7 @@ void utility_Usage(bool shortUsage)
     print_Device_Help(shortUsage, deviceHandleExample);
     print_Device_Information_Help(shortUsage);
     print_Low_Level_Info_Help(shortUsage);
+    print_SAT_Info_Help(shortUsage);
     print_Test_Unit_Ready_Help(shortUsage);
     //utility tests/operations go here
     print_Fast_Discovery_Help(shortUsage);
